;*****************************************************************************************
;  COLumnOr
;
;  Imagine color cycling without changing the palette data or the tile map.
;  This is done by adding (column_offset*16) pixels to the screen's X-position,
;  to enhance the effect the screen's x-position is changed each line to create a wave pattern. 
;  
;  Press Left or Right to decrease/increase color cycle speed!
;
;
;  All Code by -Pan- of Anthrox  (C) 1999
;  Use as You wish, modify if You want
;  For more cool, unique gags and gadgets for GameBoy visit:
;  http://www.jps.net/paninaro
;  http://www.anthrox.com
;*****************************************************************************************

SECTION "Main Mem", BSS[$C000]

xwave: ds 1
rowcounter: ds 1
xdelay: ds 1
scanoffset: ds 1
scansine: ds 2
scanline: ds 1
storage: ds 1
colsineoffset: ds 1
delayval: ds 1



;*** Vertical blanking IRQ ***
SECTION "Vertical blank IRQ",home[$0040]
 jp vbint
     
;*** LCD status IRQ ***
SECTION "LCD IRQ",home[$0048]
 jp  lcdirq
         
         



    ;******************************************************************
    ;                            Header Equates
    ;******************************************************************
;Cartridge type:
; 0 - Rom only      3 - ROM+MBC1+RAM+Battery  1 - ROM+MBC1      5 - ROM+MBC2
; 2 - ROM+MBC1+RAM  6 - ROM+MBC2+Battery
CARTTYPE  EQU 2
;Rom Size:
; 0 - 256kBit =  32kB =  2 banks ; 1 - 512kBit =  64kB =  4 banks
; 2 -   1MBit = 128kB =  8 banks ; 3 -   2MBit = 256kB = 16 banks
; 4 -   4MBit = 512kB = 32 banks
ROMSIZE   EQU 0
;Ram Size:
; 0 - None ; 1 -  16kBit =  2kB = 1 bank
; 3 - 256kBit = 32kB = 4 banks
RAMSIZE   EQU 3
;Manufacturer code (word):
MANUCODE  EQU 1
;Version Number (byte):
VERSION   EQU 1

;************************************************************************************
;     Do NOT modify unless you are familiar with header info *Modified for Game Boy Color*
;************************************************************************************

SECTION "Header",home[$0100]
  ;Jumper
  nop
  jp Start ; go to Start of program:
  ;Nintendo Scrolling Title Graphic
  DB $CE,$ED,$66,$66,$CC,$0D,$00,$0B,$03,$73,$00,$83,$00,$0C,$00,$0D
  DB $00,$08,$11,$1F,$88,$89,$00,$0E,$DC,$CC,$6E,$E6,$DD,$DD,$D9,$99
  DB $BB,$BB,$67,$63,$6E,$0E,$EC,$CC,$DD,$DC,$99,$9F,$BB,$B9,$33,$3E
  ;Title of the game
  db 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,128 ; Filled in by RGBFIX
  ;Not used
  db 0,0,0; last should be 3, won't work in emu unles it's 0
  ;Cartridge type:
  db CARTTYPE
  ;Rom Size:
  db ROMSIZE
  ;Ram Size:
  db RAMSIZE
  ;Manufacturer code:
  db 00, $33
  ;Version Number
  db VERSION
  ;Complement check
  db $0a
  ;Checksum
  dw 0





  ;*********************************************************************
  ;          This is the start of the program
  ;*********************************************************************

 ;SECTION "Main", home

Start:
  
  ld a, 01
  ldio [$4d], a
  stop             ;set to double speed!
  
  
  ld  SP,$FFF4          ; Init stack ptr
  
  di                    ; disable interrupts
  
  ld a, 0
  ldio  [$42],a         ; Y Scroll=0
  ldio  [$43],a         ; X Scroll = 0

  ld  a,%00000000       ; Turn off LCD Controller
  ldio [$40],a
 
  call copygfx          ; copy graphics to vram
  call copytiles        ; copy tiles to vram


  ld a,$80              ; set the palette at 0 and allow it to auto-increase
  ld [$68], a
   
  ld hl, colors         ; get address of color table (reads 32 colors)
  
  ld d, 64              ; # number of colors to copy * 2
copycolors:
  ld  a, [hl+]          ; get data, increase address
  ldio [$69], a         ; put the data into the palette
  dec d
  jp nz, copycolors
           


  ld     a,%10010001    ; LCD Controller = On
                        ; BG Gfx = $8000
                        ; BG Tiles   = $9800
                        ; BG     = On
  ldio    [$40],a

     
      
  ld a,%01000100	; set lcdc int
  ldio [$41],a
              
  ld a, 0
  ld [scanline],a       ; Start LCDC IRQ at line 0
  ldio  [$45],A
  
  ld a,%00000011        ; Allow LCDC Status Interrupt & Vertical Blank
  ldio  [$FF],a
  
  ei                     ; enable interrupts

  ld a,0
  ld [xwave], a
  ld [scanoffset], a
  ld [colsineoffset], a
  ld [storage], a
  ld a, 10
  ld [xdelay], a        ; this is used as a delay counter
  ld [delayval], a      ; store the speed value

;========================================================================
;                       Don't do anything important
;========================================================================  
  
megaloop:
                     ; Do Nothing
  jp megaloop        ; Do it all over again!
  



;===========================================================================================
;                              Vertical Blank Interrupt
;===========================================================================================
vbint:
   push AF
   push BC
   push DE             ; typical register saver stuff for interrupts
   push HL
    
;=========================================================
;  Read joypad, and increase / decrease color cycle speed
;=========================================================

   call joypad              ; poll joypad to read button presses
   ldio a,[$8c]
   cp $10                   ; check if Right button pressed
   jp z, increasespeed         ; if so go to speed-up routine

   cp $20                   ; check if Left button pressed
   jp z, decreasespeed      ; if so go to slow-down routine

   jp continueVb

increasespeed:
   ld a, [delayval]  
   dec a                    ; decrease the delay
   jp nz, keepit            ; if it's 0 then make it 1 (it's the minimum)
   ld a, 1
keepit:
   ld [delayval], a         ; save it
   jp continueVb
    
decreasespeed:
   ld a, [delayval]  
   inc a                    ; increase the delay
   cp $20                   ; if its $20 then make it $1f (its the maximum)
   jp nz, keepit2
   ld a, $1f
keepit2:
   ld [delayval], a   
   jp continueVb

;===============================================================================
;                 Do the sine data offset calculations
;===============================================================================


continueVb:   
   ld hl,sinedata      ; this sets up the sine data address+offset for X scrolling
   ld a,[scanoffset]
   ld e,a
   ld a,0
   ld d,a
   add hl,de
   ld a,h
   ld [scansine],a
   ld a,l
   ld [scansine+1],a
  
   ld a,[scanoffset]
   inc a                  ; increase the offset
   ld [scanoffset],a
 
        

  
  ld a, [xdelay]
  dec a                    ; without this delay the colors would flash too quickly
  ld [xdelay], a
  jp nz, skipcolormove
  ld a, [delayval]             ; this resets the delay value
  ld [xdelay], a
  
   
  ld a, [xwave]
  add $10                ; move colored columns over once
  ld [xwave], a
  

skipcolormove:  
   
   pop  HL
   pop  DE
   pop  BC    ; restore register values
   pop  AF
   reti
;===========================================================================================
;                  Routine to control each raster line via LCD Interrupt
;===========================================================================================
lcdirq:
   push AF
   push BC
   push DE              ; typical register saver stuff for interrupts
   push HL
 
  
  ld a, [storage]       ; get data we prevously stored
  ldio [$43], a         ; put it in the x-scroll register
  
  ld a,[scansine]
  ld h,a                ; fetch sine data address offset
  ld a,[scansine+1]
  ld l,a
  
  ld a, [xwave]
  ld b, a               ; get column offset
  
  ld a, [hl+]           ; get data and increase address
  add b                 ; OR it xscroll position 
  ld [storage],a        ; store it for the next line
  
  ld a, h
  ld [scansine],a             ; save address into scansine 
  ld a, l
  ld [scansine+1], a
  
  ld  a,[scanline]            ; Set next LCDC IRQ line 
  ldio  [$45],A
  ld a,[scanline]
  inc a                       ; increase the next raster line to do
  ld [scanline], a
  cp 144                      ; if in Vertical Blank then reset to 0
  jp nz, noscanlinereset
  xor a                       ; exclusive or A with A (that always means 0, same as ld a, 0)
  ld [scanline],a
  
noscanlinereset:  

  pop  HL
  pop  DE
  pop  BC    ; restore register values
  pop  AF
  reti
  
  


;***********************************************************************************
;                        Read Joypad and return values in B
;***********************************************************************************
joypad:
        LD A,$20      ; <- bit 5 = $20
        LDIO [$00],A  ; <- select P14 by setting it low
        LDIO A,[$00]
        LDIO A,[$00]  ; <- wait a few cycles
        CPL           ; <- complement A
        AND $0F       ; <- get only first 4 bits
        SWAP A        ; <- swap it
        LD B,A        ; <- store A in B
        LD A,$10
        LDIO [$00],A  ; <- select P15 by setting it low
        LDIO A,[$00]
        LDIO A,[$00]
        LDIO A,[$00]
        LDIO A,[$00]
        LDIO A,[$00]
        LDIO A,[$00]  ; <- Wait a few MORE cycles
        CPL           ; <- complement (invert)
        AND $0F       ; <- get first 4 bits
        OR B          ; <- put A and B together
                
        LD B,A        ; <- store A in B
        LDIO A,[$8B]  ; <- read old joy data from ram
        XOR B         ; <- toggle w/current button bit
        AND B         ; <- get current button bit back
        LDIO [$8C],A  ; <- save in new Joydata storage
        LD A,B        ; <- put original value in A
        LDIO [$8B],A  ; <- store it as old joy data


        LD A,$30      ; <- deselect P14 and P15
        LDIO [$00],A  ; <- RESET Joypad
        RET           ; <- Return from Subroutine
        
        ; The button values using the above method are such:
        ; read [$8C] to get the joy values:
        ; 
        ;  $8 - Start             $80 - Down
        ;  $4 - Select            $40 - Up
        ;  $2 - B                 $20 - Left
        ;  $1 - A                 $10 - Right
        



;***************************************************************
;*                          Copy Tile Gfx
;***************************************************************

copygfx:
 ld  bc,colgfx
 ld  hl,$8000
 ld  de, 16          ;# of bytes to copy

copyloop:
 ldio  a,[$41]       ; Wait until you can write to vram
 and 2              
 jr  nz, copyloop   
     
 ld  a, [bc]
 ld  [hli],a
 inc  bc
 dec  de
 ld a,d
 cp $ff
 jp  nz,copyloop
 ret



;***************************************************************
;*                          Copy Tile Map
;***************************************************************

copytiles:
   
  ld  hl,$9800
  ld  de, 1024          ;# of bytes to erase
eraseloop:

 ldio  a,[$41]          ; Wait until you can write to vram
 and  2              
 jr nz,eraseloop      
 ld  a,$00

 ld  [hli],a
 dec  de
 ld  a,d
 cp  $ff
 jp  nz,eraseloop
 
 
  ld a, 1
  ldio [$4f], a          ; switch to VRAM bank 1 so we can add palette and x flip data!
  
  ld  bc, paletternd     ; get address of palette data
  ld  a, 0
  ld  [rowcounter], a    ; # of rows that were made
  ld  hl,$9800           ; vram address
  ld  d, $00             ; # of columns that were made
createloop:

  ldio  a,[$41]          ; Wait until you can write to vram
  and  2              
  jr nz,createloop      

  ld a, d                ; get column
  and 1                  ; if it's the right side of the column the we need to x-flip the tile
  scf
  ccf
  rl a                   ; take that bit and shift it 5 times to make it the X-flip bit
  rl a
  rl a
  rl a
  rl a
  
  ld e, a                ; store it in E
  ld  a, [bc]            ; get palette #
  or e                   ; add the x-flip bit 
  ld  [hli],a            ; put it in Vram
  
  ld a, d                ; get column
  and 1
  jp z, colornevermind   ; did we do the right side of the column? (each column is 2 tiles
                         ; made up of 1 tile, the right one is flipped horizontally)
                         ; if we didn't make the right side yet then don't change the palette
                         ; address!
  inc bc                 ; increase palette address
colornevermind:
  inc d                  ; increase the column
  ld a, d
  cp 32                  ; compare it with 32 (number of tiles to be made)
  jp nz, createloop      ; if not then go back
  ld d, 0                ; if so then reset column to 0 
  ld bc, paletternd      ; reset palette address to beginning
  ld a, [rowcounter]
  inc a                  ; increase the row we're on
  ld [rowcounter], a
  cp 32                  ; see if it's 32 rows
  jp nz, createloop      ; if not the go back again
  

  ld a, 0
  ldio [$4f], a          ; switch to VRAM bank 1
   
 ret




;=============================================================================
;                          The rest is all data
;=============================================================================
 

paletternd:
db  7,  1,  3,  2,  0,  4,  5,  6
db  5,  7,  1,  2,  6,  4,  3,  0

sinedata:
db -8, -9, -9, -10, -11, -12, -12, -14
db -15, -16, -17, -18, -20, -21, -22, -23
db -24, -25, -25, -26, -26, -27, -27, -27
db -27, -26, -26, -25, -25, -24, -23, -22
db -21, -20, -19, -18, -18, -17, -16, -16
db -16, -15, -15, -16, -16, -16, -17, -18
db -19, -20, -21, -22, -24, -25, -26, -28
db -29, -31, -32, -33, -34, -35, -36, -37
db -38, -38, -39, -39, -39, -39, -39, -39
db -39, -39, -38, -38, -38, -37, -37, -37
db -36, -36, -36, -36, -36, -37, -37, -37
db -38, -39, -39, -40, -41, -42, -43, -44
db -45, -45, -46, -47, -47, -48, -48, -48
db -48, -47, -47, -46, -45, -44, -42, -41
db -39, -37, -36, -34, -31, -29, -27, -25
db -23, -21, -19, -17, -15, -13, -12, -10
db -9, -8, -7, -6, -5, -5, -4, -4
db -4, -4, -4, -4, -4, -4, -4, -4
db -3, -3, -3, -2, -2, -1,  0,  1
db  2,  4,  5,  7,  8,  10,  12,  14
db  15,  17,  19,  21,  22,  24,  25,  27
db  28,  29,  30,  30,  31,  31,  31,  31
db  31,  31,  30,  30,  29,  29,  28,  27
db  26,  25,  25,  24,  23,  22,  22,  22
db  21,  21,  21,  21,  21,  21,  21,  21
db  22,  22,  22,  22,  23,  23,  23,  23
db  23,  23,  22,  22,  21,  20,  19,  18
db  17,  15,  14,  12,  10,  9,  7,  5
db  3,  1, -1, -3, -5, -6, -8, -9
db -10, -12, -12, -13, -14, -14, -14, -14
db -14, -14, -14, -13, -13, -12, -11, -11
db -10, -10, -9, -9, -8, -8, -8, -8
db -8, -9, -9, -10, -11, -12, -12, -14
db -15, -16, -17, -18, -20, -21, -22, -23
db -24, -25, -25, -26, -26, -27, -27, -27
db -27, -26, -26, -25, -25, -24, -23, -22
db -21, -20, -19, -18, -18, -17, -16, -16
db -16, -15, -15, -16, -16, -16, -17, -18
db -19, -20, -21, -22, -24, -25, -26, -28
db -29, -31, -32, -33, -34, -35, -36, -37
db -38, -38, -39, -39, -39, -39, -39, -39
db -39, -39, -38, -38, -38, -37, -37, -37
db -36, -36, -36, -36, -36, -37, -37, -37
db -38, -39, -39, -40, -41, -42, -43, -44
db -45, -45, -46, -47, -47, -48, -48, -48
db -48, -47, -47, -46, -45, -44, -42, -41
db -39, -37, -36, -34, -31, -29, -27, -25
db -23, -21, -19, -17, -15, -13, -12, -10
db -9, -8, -7, -6, -5, -5, -4, -4
db -4, -4, -4, -4, -4, -4, -4, -4
db -3, -3, -3, -2, -2, -1,  0,  1
db  2,  4,  5,  7,  8,  10,  12,  14
db  15,  17,  19,  21,  22,  24,  25,  27
db  28,  29,  30,  30,  31,  31,  31,  31
db  31,  31,  30,  30,  29,  29,  28,  27
db  26,  25,  25,  24,  23,  22,  22,  22
db  21,  21,  21,  21,  21,  21,  21,  21
db  22,  22,  22,  22,  23,  23,  23,  23
db  23,  23,  22,  22,  21,  20,  19,  18
db  17,  15,  14,  12,  10,  9,  7,  5
db  3,  1, -1, -3, -5, -6, -8, -9
db -10, -12, -12, -13, -14, -14, -14, -14
db -14, -14, -14, -13, -13, -12, -11, -11
db -10, -10, -9, -9, -8, -8, -8, -8




colgfx:                                    ; just one character (1 tile 8*8)
DB $EE,$F0,$C4,$F8,$EE,$F0,$C4,$F8
DB $EE,$F0,$C4,$F8,$EE,$F0,$C4,$F8

colors:
DB $FF,$7F, $9F,$52, $5F,$29, $1F,$00, $FF,$7F, $94,$7E, $4A,$7D, $00,$7C
DB $FF,$7F, $94,$53, $20,$2B, $60,$02, $FF,$7F, $7D,$53, $1B,$2B, $9A,$02
DB $FF,$7F, $9F,$7E, $5F,$7D, $1F,$7C, $FF,$7F, $74,$6F, $0A,$5F, $80,$52
DB $FF,$7F, $7B,$6F, $F7,$5E, $94,$52, $FF,$7F, $9F,$6A, $5F,$55, $1F,$44

